unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls, EditBtn, Printers;

type

  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    ComboBox1: TComboBox;
    FileNameEdit1: TFileNameEdit;
    Image1: TImage;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Panel1: TPanel;
    RadioGroup1: TRadioGroup;
    procedure Button1Click(Sender: TObject);
    procedure ComboBox1Change(Sender: TObject);
    procedure FileNameEdit1AcceptFileName(Sender: TObject; var Value: String);
    procedure FormCreate(Sender: TObject);
    procedure Image1MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure Image1MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure Image1MouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure Image1Paint(Sender: TObject);
    procedure RadioGroup1Click(Sender: TObject);
  private

  public
    FSelecting: Boolean;
    FSelection: TRect;
    procedure GeneratePreview;
    procedure PrintImage;
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

procedure TForm1.Button1Click(Sender: TObject);
begin
  PrintImage;
end;

procedure TForm1.ComboBox1Change(Sender: TObject);
begin
  Printer.PrinterIndex := ComboBox1.ItemIndex;
  Label4.Caption := Printer.PaperSize.PaperName;
  RadioGroup1Click(Sender);
end;

procedure TForm1.FileNameEdit1AcceptFileName(Sender: TObject; var Value: String);
begin
  Image1.Picture.LoadFromFile(Value);
  GeneratePreview;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  ComboBox1.Items := Printer.Printers;
  if ComboBox1.Items.Count > 0 then
  begin
    ComboBox1.ItemIndex := 0;
    ComboBox1Change(Sender);
  end;
end;

procedure TForm1.Image1MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then
  begin
    FSelection.Left := X;
    FSelection.Top := Y;
    FSelecting := True;
  end;
end;

procedure TForm1.Image1MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
  if FSelecting then
  begin
    FSelection.Right := X;
    FSelection.Bottom := Y;
    Image1.Invalidate;
  end;
end;

procedure TForm1.Image1MouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if FSelecting then
  begin
    FSelecting := False;
    FSelection.Right := X;
    FSelection.Bottom := Y;
    Image1.Invalidate;
    GeneratePreview;
  end;
end;

procedure TForm1.Image1Paint(Sender: TObject);
begin
  Image1.Canvas.Brush.Style := bsClear;
  Image1.Canvas.Pen.Width := 3;
  Image1.Canvas.Pen.Color := clYellow;
  Image1.Canvas.Rectangle(FSelection);
end;

procedure TForm1.RadioGroup1Click(Sender: TObject);
begin
  if RadioGroup1.ItemIndex = 1 then
    Printer.Orientation := poLandscape
  else
    Printer.Orientation := poPortrait;
  GeneratePreview;
end;

procedure TForm1.GeneratePreview;
var
  bmp, img: TBitmap;
  pagescale: Double = 0.3;
  imgscale: Double;
  pw, ph, w, h, mx, my: Integer;
  imgr: TRect;
begin
  panel1.Repaint;
  bmp := TBitmap.Create;
  //get page size
  pw := trunc(Printer.PaperSize.Width / Printer.XDPI * 72 * pagescale);
  ph := trunc(Printer.PaperSize.Height / Printer.YDPI * 72 * pagescale);
  //get page size without margins
  w := trunc(Printer.PaperSize.PaperRect.WorkRect.Width / Printer.XDPI * 72 * pagescale);
  h := trunc(Printer.PaperSize.PaperRect.WorkRect.Height / Printer.YDPI * 72 * pagescale);
  //margins
  mx := (pw - w) div 2;
  my := (ph - h) div 2;
  bmp.SetSize(pw, ph);
  bmp.Canvas.Brush.Color := clWhite;
  bmp.Canvas.FillRect(0, 0, pw, ph);
  //get image to print, if no selection then the whole picture
  if (Image1.Picture.Bitmap.Width > 0) and (Image1.Picture.Bitmap.Height > 0) then
  begin
    img := TBitmap.Create;
    if (FSelection.Width = 0) or (FSelection.Height = 0) then
    begin
      img.SetSize(Image1.Picture.Width, Image1.Picture.Height);
      img.Canvas.Draw(0, 0, Image1.Picture.Bitmap);
    end
    else   //otherwise only the selected area
    begin
      img.SetSize(abs(FSelection.Width), abs(FSelection.Height));
      img.Canvas.CopyRect(Rect(0, 0, img.Width, img.Height), Image1.Picture.Bitmap.Canvas, FSelection);
    end;
    //center image on page
    imgscale := (w / img.Width);
    if (h / img.Height) < imgscale then
      imgscale := (h / img.Height);
    imgr.Left := mx + (w - trunc(img.Width * imgscale)) div 2;
    imgr.Top := my + (h - trunc(img.Height * imgscale)) div 2;
    imgr.Width := trunc(img.Width * imgscale);
    imgr.Height := trunc(img.Height * imgscale);
    bmp.Canvas.StretchDraw(imgr, img);
    img.Free;
  end;
  //preview
  panel1.Canvas.Draw(0, 0, bmp);
  bmp.Free;
end;

procedure TForm1.PrintImage;
var
  img: TBitmap;
  imgscale: Double;
  w, h, mx, my: Integer;
  imgr: TRect;
begin
  if (Image1.Picture.Bitmap.Width = 0) or (Image1.Picture.Bitmap.Height = 0) then
  begin
    ShowMessage('Select image');
    exit;
  end;
  //get page size without margins
  w := Printer.PaperSize.PaperRect.WorkRect.Width;
  h := Printer.PaperSize.PaperRect.WorkRect.Height;
  //margins
  mx := (Printer.PaperSize.Width - w) div 2;
  my := (Printer.PaperSize.Height - h) div 2;
  //get image to print, if no selection then the whole picture
  img := TBitmap.Create;
  if (FSelection.Width = 0) or (FSelection.Height = 0) then
  begin
    img.SetSize(Image1.Picture.Width, Image1.Picture.Height);
    img.Canvas.Draw(0, 0, Image1.Picture.Bitmap);
  end
  else   //otherwise only the selected area
  begin
    img.SetSize(abs(FSelection.Width), abs(FSelection.Height));
    img.Canvas.CopyRect(Rect(0, 0, img.Width, img.Height), Image1.Picture.Bitmap.Canvas, FSelection);
  end;
  //center image on page
  imgscale := (w / img.Width);
  if (h / img.Height) < imgscale then
    imgscale := (h / img.Height);
  imgr.Left := mx + (w - trunc(img.Width * imgscale)) div 2;
  imgr.Top := my + (h - trunc(img.Height * imgscale)) div 2;
  imgr.Width := trunc(img.Width * imgscale);
  imgr.Height := trunc(img.Height * imgscale);
  //print
  Printer.Title := 'Test';
  Printer.BeginDoc;
  Printer.Canvas.StretchDraw(imgr, img);
  Printer.EndDoc;
  img.Free;
end;

end.
